using Microsoft.Extensions.Logging;
using ProjectTank.CollectionGenericObjects;
using ProjectTank.Drawning;
using ProjectTank.Drawnings;
using ProjectTank.Exceptions;

namespace ProjectTank
{
    /// <summary>
    /// Форма работы с компанией и её коллекцией
    /// </summary>
    public partial class FormBattleTankCollection : Form
    {
        /// <summary>
        /// Хранилище коллекций
        /// </summary>
        private readonly StorageCollection<DrawningTank> _storageCollection;


        /// <summary>
        /// Компания
        /// </summary>
        private AbstractCompany? _company = null;

        private readonly ILogger _logger;
        /// <summary>
        /// Конструктор
        /// </summary>
        public FormBattleTankCollection(ILogger<FormBattleTankCollection> logger)
        {
            InitializeComponent();
            _storageCollection = new();
            _logger = logger;
            _logger.LogInformation("Форма загрузилась");
        }

        /// <summary>
        /// Выбор компании
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ComboBoxSelectorCompany_SelectedIndexChanged(object sender, EventArgs e)
        {
            panelCompanyTools.Enabled = false;
        }

        /// <summary>
        /// Создание объекта класса-перемещения
        /// </summary>
        /// <param name="type">Тип создоваемого объекта</param>
        private void SetTank(DrawningTank tank)
        {
            if (_company == null || tank == null)
            {
                return;
            }
            try
            {
                if (_company + tank >= 0)
                {
                    MessageBox.Show("Объект добавлен");
                    pictureBox.Image = _company.Show();
                    _logger.LogInformation("Объект добавлен: " + tank.GetDataForSave());
                }
            }
            catch (CollectionOverflowException ex)
            {
                MessageBox.Show(ex.Message);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
            catch (ObjectNotFoundException ex)
            {
                MessageBox.Show(ex.Message);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
            catch (ObjectAlreadyExistsException)
            {
                MessageBox.Show("Такой объект уже существует");
                _logger.LogError("Ошибка: такой объект уже существует {0}", tank);
            }
        }

        /// <summary>
        /// добавление танка
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonAddTank_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }

            FormTankConfig form = new();
            form.AddEvent(SetTank);
            form.Show();
        }


        private void buttonRemoveTank_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(maskedTextBox.Text) || _company == null)
            {
                return;
            }

            if (MessageBox.Show("Удалить объект?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
            {
                return;
            }
            int pos = Convert.ToInt32(maskedTextBox.Text);
            try
            {
                if (_company - pos != null)
                {
                    MessageBox.Show("Объект удален");
                    pictureBox.Image = _company.Show();
                    _logger.LogInformation("Объект удален, позиция: " + pos);
                }
            }
            catch (ObjectNotFoundException ex)
            {
                MessageBox.Show(ex.Message);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
            catch (PositionOutOfCollectionException ex)
            {
                MessageBox.Show(ex.Message);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
        /// <summary>
        /// передать на тесты
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonGoToCheck_Click(object sender, EventArgs e)
        {
            if (_company == null)
            {
                return;
            }
            DrawningTank? tank = null;
            int counter = 100;
            while (tank == null)
            {
                try
                {
                    tank = _company.GetRandomObject();
                    counter--;
                    if (counter <= 0)
                    {
                        break;
                    }
                }
                catch (ObjectNotFoundException ex)
                {
                    MessageBox.Show(ex.Message);
                    _logger.LogError("Ошибка: {Message}", ex.Message);
                }
                catch (PositionOutOfCollectionException ex)
                {
                    MessageBox.Show(ex.Message);
                    _logger.LogError("Ошибка: {Message}", ex.Message);
                }
            }
            if (tank == null)
            {
                return;
            }
            FormBattleTank form = new()
            {
                SetTank = tank
            };
            form.ShowDialog();
        }
        /// <summary>
        /// обновление
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonRefresh_Click(object sender, EventArgs e)
        {
            if (_company == null) return;
            pictureBox.Image = _company.Show();
        }
        /// <summary>
        ///  Добавление коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCollectionAdd_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(textBoxCollectionName.Text) || (!radioButtonList.Checked && !radioButtonMassive.Checked))
            {
                MessageBox.Show("Не все данные заполнены", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            try
            {
                CollectionType collectionType = CollectionType.None;
                if (radioButtonMassive.Checked)
                {
                    collectionType = CollectionType.Massive;
                }
                else if (radioButtonList.Checked)
                {
                    collectionType = CollectionType.List;
                }
                _storageCollection.AddCollection(textBoxCollectionName.Text, collectionType);
                RerfreshListBoxItems();
                _logger.LogInformation("Коллекция добавлена: " + textBoxCollectionName.Text);
            }
            catch (Exception ex)
            {
                //MessageBox.Show(ex.Message);
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }

        /// <summary>
        /// Удвление коллекции
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void buttonCollectionDel_Click(object sender, EventArgs e)
        {
            if (!radioButtonList.Checked && !radioButtonMassive.Checked || string.IsNullOrEmpty(textBoxCollectionName.Text))
            {
                return;
            }
            try
            {
                if (MessageBox.Show("Удалить коллекцию?", "Удаление", MessageBoxButtons.YesNo, MessageBoxIcon.Question) != DialogResult.Yes)
                {
                    return;
                }
                _storageCollection.DelCollection(listBoxСollection.SelectedItem.ToString());
                RerfreshListBoxItems();
                _logger.LogInformation("Коллекция удалена: " + listBoxСollection.SelectedItem.ToString());
            }
            catch (Exception ex)
            {
                _logger.LogError("Ошибка: {Message}", ex.Message);
            }
        }
        /// <summary>
        /// Обновление списка в listBoxCollection
        /// </summary>
        private void RerfreshListBoxItems()
        {
            listBoxСollection.Items.Clear();
            for (int i = 0; i < _storageCollection.Keys?.Count; ++i)
            {
                string? colName = _storageCollection.Keys?[i].Name;
                if (!string.IsNullOrEmpty(colName))
                {
                    listBoxСollection.Items.Add(colName);
                }
            }

        }
        /// <summary>
        ///  Создание компании
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void ButtonCreateCompany_Click(object sender, EventArgs e)
        {
            if (listBoxСollection.SelectedIndex < 0 || listBoxСollection.SelectedItem == null)
            {
                MessageBox.Show("Коллекция не выбрана");
                return;
            }

            ICollectionGenericObjects<DrawningTank>? collection = _storageCollection[listBoxСollection.SelectedItem.ToString() ?? string.Empty];
            if (collection == null)
            {
                MessageBox.Show("Коллекция не проинициализирована");
                return;
            }

            switch (comboBoxSelectorCompany.Text)
            {
                case "Хранилище":
                    _company = new TankBase(pictureBox.Width, pictureBox.Height, collection);
                    break;
            }

            panelCompanyTools.Enabled = true;
            RerfreshListBoxItems();
        }
        private void SaveToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (saveFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storageCollection.SaveData(saveFileDialog.FileName);
                    MessageBox.Show("Сохранение прошло успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    _logger.LogInformation("Сохранение в файл: {filename}", saveFileDialog.FileName);

                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogError("Ошибка: {Message}", ex.Message);
                }
            }
        }

        /// <summary>
        /// Обработка нажатия "Загрузка"
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void LoadToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (openFileDialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _storageCollection.LoadData(openFileDialog.FileName);
                    MessageBox.Show("Загрузка прошла успешно", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    RerfreshListBoxItems();
                    _logger.LogInformation("Загрузка из файла: {filename}", openFileDialog.FileName);
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message, "Результат", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    _logger.LogError("Ошибка: {Message}", ex.Message);
                }
            }
        }

        private void buttonSortByType_Click(object sender, EventArgs e)
        {
            CompareTank(new DrawningTankCompareByType());
        }

        private void buttonSortByColor_Click(object sender, EventArgs e)
        {
            CompareTank(new DrawningTankCompareByColor());
        }
        private void CompareTank(IComparer<DrawningTank?> comparer)
        {
            if (_company == null)
            {
                return;
            }
            _company.Sort(comparer);
            pictureBox.Image = _company.Show();
        }
    }
}